/* =========================================================================
	This extention is part of IshiSoft CEHTTP.
	------------------------------------------------------------------------
	author: yu-ishi@ya2.so-net.ne.jp
			http://www001.upp.so-net.ne.jp/ishi/index.html
  ========================================================================== */
#include "stdafx.h"
#include <stdio.h>	// for PCHTTP
#include <stdlib.h>	// for PCHTTP
#include "MyHash.h"

#define MAX_RECORD_LEN	2048

CRITICAL_SECTION cs;
char* g_pszFormat = NULL;
const char* g_pNullString = "-";
const char* g_pszMonth[12] =
{
	"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
};

static const char* GetValueAvoidNull( MyHash* pHashHeaders, char* pszKey );
static char *STRNCPY( char *dst, const char *src, size_t count );
static unsigned int STRLEN( const char *string );

//-----------------------------------------------------------------------------
//	@detail
//		Create logfile extention
//		work on CEHTTP directory.
//	@param
//		WCHAR* pwszLogFile		// File to create
//		int nLogSize			// Limit of logfile
//		int nHttpStatus			// http status
//		MyHash* pHashHeaders	// headers
//	@return
//	@comment
//-----------------------------------------------------------------------------
int fnCehttpLog( WCHAR* pwszLogFile, int nLogSize, int nHttpStatus, MyHash* pHashHeaders )
{
	HANDLE hFile = INVALID_HANDLE_VALUE;
	DWORD dwWritten;
	int	nRecordLen = 0;

	if( NULL == g_pszFormat ) return 0;

	EnterCriticalSection( &cs );

	// ------------------------------------------------------------------------
	SYSTEMTIME st;
	GetLocalTime(&st);

	const char* pRemoteHost = pHashHeaders->GetValueI( "REMOTE_HOST" );
	if( NULL == pRemoteHost )
	{
		pRemoteHost	= GetValueAvoidNull( pHashHeaders, "remote_addr" );
	}
	const char* pRequestLine	= GetValueAvoidNull( pHashHeaders, "request_line" );
	const char* pSendBytes		= GetValueAvoidNull( pHashHeaders, "send_bytes" );
	const char* pReferer		= GetValueAvoidNull( pHashHeaders, "referer" );
	const char* pUserAgent		= GetValueAvoidNull( pHashHeaders, "user-agent" );
	const char* pAuthName		= GetValueAvoidNull( pHashHeaders, "authentication_user" );

	nRecordLen = STRLEN(pRemoteHost)
				+ STRLEN(pRequestLine)
				+ STRLEN(pReferer)
				+ STRLEN(pUserAgent)
				+ STRLEN(pAuthName);

	// ------------------------------------------------------------------------
	// create file
	// ------------------------------------------------------------------------
	hFile = CreateFile( pwszLogFile, GENERIC_WRITE, 0, NULL, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
	
	if( hFile == INVALID_HANDLE_VALUE )
	{
		LeaveCriticalSection( &cs );
		return 0;
	}

	if( 0xFFFFFFFF == SetFilePointer( hFile, 0, 0, FILE_END ) )
	{
		LeaveCriticalSection( &cs );
		CloseHandle( hFile );
		return 0;
	}


	if( nRecordLen < MAX_RECORD_LEN - 512 )
	{
		sprintf( g_pszFormat, 
			"%s - %s [%02d/%s/%4d:%02d:%02d:%02d +0900] \"%s\" %d %s \"%s\" \"%s\"\n"
			, pRemoteHost
			, pAuthName
			, st.wDay, g_pszMonth[st.wMonth-1], st.wYear
			, st.wHour, st.wMinute, st.wSecond
			, pRequestLine
			, nHttpStatus
			, pSendBytes
			, pReferer
			, pUserAgent );
	}
	else
	{
		STRNCPY( g_pszFormat, pRemoteHost, MAX_RECORD_LEN - 1 );
	}

	WriteFile( hFile, g_pszFormat, STRLEN(g_pszFormat), &dwWritten, NULL );

	DWORD dwFileSize = GetFileSize( hFile, NULL ); 

	// ------------------------------------------------------------------------
	// close file
	// ------------------------------------------------------------------------

	CloseHandle( hFile );
	hFile = INVALID_HANDLE_VALUE;

	// ------------------------------------------------------------------------
	// change backup if necessary
	// ------------------------------------------------------------------------

	if( dwFileSize > (DWORD)nLogSize )		// original file size reached max size
	{
		WCHAR	wszBackUp[MAX_PATH] = {0};
		int nLen;
		
		nLen = wcslen( pwszLogFile );
		wcscpy( wszBackUp, pwszLogFile );

		wszBackUp[nLen-1] = '_';				// last word of original file is changed to '_'
		DeleteFile( wszBackUp );				// delete backup file
		MoveFile( pwszLogFile, wszBackUp );	// rename original file to backup file
	}

	LeaveCriticalSection( &cs );	// --->>>

	// ------------------------------------------------------------------------
	return 1;
}

static const char* GetValueAvoidNull( MyHash* pHashHeaders, char* pszKey )
{
	char* p =  pHashHeaders->GetValueI( pszKey );
	if( p == NULL )
		return g_pNullString;
	return p;
}

static char *STRNCPY( char *dst, const char *src, size_t count )
{
	size_t ii;
	int flag;
	char* p = dst;

	for( ii = 0, flag = 0; ii < count; ii++ )
	{
		if( *src == '\0' )
			flag = 1;

		*dst++ = ( flag ? '\0' : *src++ );
	}

	return p;
}

static unsigned int STRLEN( const char *string )
{
	if( string == NULL ) return 0;
	for( unsigned int ii = 0; *string != '\0'; ii++, string++ );
	return ii;
}


BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
    switch( ul_reason_for_call )
	{
	case DLL_PROCESS_ATTACH:
		InitializeCriticalSection( &cs );
		g_pszFormat = (char*)MALLOC( MAX_RECORD_LEN );
		if( NULL == g_pszFormat )	return FALSE;
		break;
	case DLL_THREAD_ATTACH:
		break;
	case DLL_THREAD_DETACH:
		break;
	case DLL_PROCESS_DETACH:
		DeleteCriticalSection( &cs );
		FREE( g_pszFormat );
		break;
	default:
		break;
	}
    return TRUE;
}

